#include "mainwindow.h"
#include "ui_mainwindow.h"

int MainWindow::serial_ready = 0;
MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    //只能输入整形数据
    ui->editSetSpeed->setValidator(new QIntValidator(ui->editSetSpeed));
    tim = new QTimer();
    tim->setInterval(1000);
    connect(tim,SIGNAL(timeout()),this,SLOT(onTimeOut()));
    tim->start();
    //读取串口信息
        foreach (const QSerialPortInfo &info, QSerialPortInfo::availablePorts())
        {
            qDebug()<<"Name:"<<info.portName();
            qDebug()<<"Description:"<<info.description();
            qDebug()<<"Manufacturer:"<<info.manufacturer();

            //这里相当于自动识别串口号之后添加到了cmb，如果要手动选择可以用下面列表的方式添加进去
            QSerialPort serial;
            serial.setPort(info);
            if(serial.open(QIODevice::ReadWrite))
            {
                //将串口号添加到cmb
                ui->cmbPortName->addItem(info.portName());
                //关闭串口等待人为(打开串口按钮)打开
                serial.close();
            }
        }
        ui->openMotorBut->setEnabled(false);
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::on_openSerBut_clicked()
{
    ui->textBrowser->insertPlainText("clicke openserialbutton!\n");
    if(ui->openSerBut->text()=="打开串口")
    {

        serialPort = new QSerialPort;
        serialPort->setPortName(ui->cmbPortName->currentText());

        if(!serialPort->open(QIODevice::ReadWrite))
        {
            QMessageBox::about(NULL, "错误",
            "串口无法打开！可能串口已经被占用！");
        }
        else {
            ui->openSerBut->setText("关闭串口");
            ui->openMotorBut->setEnabled(false);
        }
        serialPort->setBaudRate(QSerialPort::Baud19200);
        serialPort->setDataBits(QSerialPort::Data8);
        serialPort->setParity(QSerialPort::NoParity);
        serialPort->setStopBits(QSerialPort::OneStop);

        ui->openSerBut->setText("关闭串口");
        serial_ready = 1;
        connect(serialPort, SIGNAL(readyRead()),
            this, SLOT(serialPortReadyRead()));
        ui->openMotorBut->setEnabled(true);
    }
    else{
        serial_ready = 0;
        serialPort->clear();
        serialPort->close();
        serialPort->deleteLater();
         ui->openSerBut->setText("打开串口");
         ui->openMotorBut->setEnabled(false);
    }
}

void MainWindow::serialPortReadyRead()
 {
     /* 接收缓冲区中读取数据 */
    quint16 speed;
     QByteArray buf = serialPort->readAll();
     if(buf[0] == 0x01 && buf[1] == 0x03 && buf[2] == 0x02)
     {
         speed = buf[3]<<8 | buf[4];
         speed = speed/10;
         ui->lcdNumber->display(QString::number(speed));
     }
     ui->textBrowser->insertPlainText(QString(buf.toHex()));
 }


quint16 MainWindow::CalcCRC16(quint8 *SC_Buffer, quint8 SC_Amount)//生成校验码
{
    quint16 Crc;
    quint8 n, m, x;

    Crc= 0xFFFF;//16个1
    m= SC_Amount;//拿到长度
    x= 0;

    while(m>0){
        Crc ^= SC_Buffer[x];//第一个数据帧异或
        for(n=0; n<8; n++){//因为是quint8是8位数据，所以循环8次
            if(Crc & 1){//移出位为1，就进行异或
                Crc >>= 1;
                Crc ^= 0xA001;
            }else//为0继续移出
                Crc >>= 1;
        }
        m--;//下一个数据帧
        x++;
    }

    x = Crc>>8;//拿到高8位
    Crc <<= 8;//剩下低8位
    Crc &= 0xff00;//低位清0
    Crc |= x;//实现高低位位置互换

    return Crc;
}





void MainWindow::on_openMotorBut_clicked()
{
    QByteArray data;
    data.resize(8);
    quint8 crc[8];
    quint8 n;

    ui->textBrowser->insertPlainText("start clicked!\n");
    if(ui->openMotorBut->text() == "启动电机")
    {
        //EN set
        crc[0] = 0x01;
        crc[1] = 0x06;
        crc[2] = 0x00;
        crc[3] = 0x00;
        crc[4] = 0x00;
        crc[5] = 0x01;
        quint16 reCrc;
        reCrc = CalcCRC16(crc,6);
        crc[6] = reCrc>>8;
        crc[7] = reCrc & 0xff;

        for(n=0;n<8;n++){
            data[n] = crc[n];
        }
        serialPort->write(data);
        ui->openMotorBut->setText("关闭电机");
    }
    else {
        //close
        crc[0] = 0x01;
        crc[1] = 0x06;
        crc[2] = 0x00;
        crc[3] = 0x00;
        crc[4] = 0x00;
        crc[5] = 0x01;
        quint16 reCrc;
        reCrc = CalcCRC16(crc,6);
        crc[6] = reCrc>>8;
        crc[7] = reCrc & 0xff;
        ui->openMotorBut->setText("启动电机");
    }
}

void MainWindow::on_setSpeedBut_clicked()
{
    QByteArray data;
    data.resize(8);
    quint8 crc[8];
    quint8 n;
    quint16 speed_val;
    ui->textBrowser->insertPlainText("set speed!\n");
        //speed set 01 06 00 02 05 DC 2A C3
    speed_val = ui->editSetSpeed->text().toInt();
    if (speed_val>3000)
    {
        speed_val = 3000;
    }
    crc[0] = 0x01;
    crc[1] = 0x06;
    crc[2] = 0x00;
    crc[3] = 0x02;

    crc[4] = speed_val>>8;
    crc[5] = speed_val & 0xff;
    quint16 reCrc;
    reCrc = CalcCRC16(crc,6);
    crc[6] = reCrc>>8;
    crc[7] = reCrc & 0xff;

    for(n=0;n<8;n++){
        data[n] = crc[n];
    }
    serialPort->write(data);
    ui->textBrowser->setText("set speed val");

}

void MainWindow::on_cleanBut_clicked()
{
    ui->textBrowser->clear();
}

void MainWindow::read_speed()
{
    QByteArray data;
    quint8 n;
    quint8 crc[8] = {0x01, 0x03, 0x00, 0x10, 0x00, 0x01, 0x85, 0xcf};
    for(n=0;n<8;n++){
        data[n] = crc[n];
    }
    serialPort->write(data);
}
void MainWindow::onTimeOut()
{
    if (serial_ready == 1)
    {
        read_speed();
    }
}
